#include <SFML/Graphics.hpp>

constexpr int WIDTH(400);
constexpr int HEIGHT(400);
constexpr int ESCAPE_MAX(500);
constexpr double GRAPHX_MAX(2.0);
constexpr double GRAPHX_MIN(-0.5);
constexpr double GRAPHY_MAX(1.0);
constexpr double GRAPHY_MIN(-1.0);

void generate_mandelbrot_set(int width, int height, sf::VertexArray& array);
double pixel_to_coord(int pixelCoord, double graphWidth, int imageWidth, double graphMin);
int compute_escape_value(double real, double imaginary, int escapeMax);
 
int main()
{
	
	sf::VertexArray varray;
 
    sf::RenderWindow window(sf::VideoMode(WIDTH, HEIGHT), "Mandelbrot");
	generate_mandelbrot_set(WIDTH, HEIGHT, varray);
 
    while (window.isOpen())
    {
        sf::Event event;
        while (window.pollEvent(event))
        {
            if (event.type == sf::Event::Closed)
                window.close();
        }

		window.clear();
        window.draw(varray);
        window.display();
    }
	return EXIT_SUCCESS;

}
 
void generate_mandelbrot_set(int width, int height, sf::VertexArray& varray)
{
    varray.clear();
    double graphX;
    double  graphY;
    int x, y, escape;
    sf::Vertex vertex;
    for (y = 0; y < height; y++)
    {
        graphY = pixel_to_coord(y, GRAPHY_MAX - GRAPHY_MIN, height, GRAPHY_MIN);
 
        for (x = 0; x < width; x++)
        {
            vertex.position = sf::Vector2f(x, y);
            graphX = pixel_to_coord(x, GRAPHX_MAX - GRAPHX_MIN, width, GRAPHX_MIN);
            escape = compute_escape_value(graphX, graphY, ESCAPE_MAX);
             
            if (escape == ESCAPE_MAX)
            {
                vertex.color = sf::Color::Green;
            }
            else
            {
                vertex.color = sf::Color(0,102,102);
            }
 
            varray.append(vertex);
        }
    }
}
 
double pixel_to_coord(int pixelCoord, double graphWidth, int imageWidth, double graphMin)
{
    return ((double)pixelCoord * (graphWidth / (double)imageWidth)) + graphMin;
}
 
int compute_escape_value(double real, double imaginary, int escapeMax)
{
    double zReal, zImag;
    double z0Real, z0Imag;
    int n, escape;
    zReal = zImag = 0;
    z0Real = z0Imag = 0;
 
    for (n = 0; n <= escapeMax; n++)
    {
        z0Real = ((zReal * zReal) - (zImag * zImag) - real);
        z0Imag = (2 * (zReal * zImag)) + imaginary;
 
        if (((z0Real * z0Real) + (z0Imag * z0Imag)) >= 4)
        {
            escape = n;
            return escape;
        }
 
        zReal = z0Real;
        zImag = z0Imag;
        escape = escapeMax;
    }
     
    return escape;
}
