#include <SFML/Graphics.hpp>
#include <iostream>
#include <cmath>
#include "Config.hpp"

void drawLine(sf::RenderTarget& target, double x1, double y1, double x2, double y2)
{
		sf::VertexArray heading(sf::PrimitiveType::Lines, 2);
		heading[0] = { sf::Vector2f(x1, y1), sf::Color::White};
		heading[1] = { sf::Vector2f(x2, y2), sf::Color::White };
		target.draw(heading);
}

void ligne_brisee(sf::RenderTarget& target, double x1, double y1, double x2, double y2)
{
	double e = 2.0 * sqrt(3.0);

	double
		x1p = (2.0 * x1 + x2) / 3.0,
		y1p = (2.0 * y1 + y2) / 3.0,
		x2p = (x1 + x2) / 2.0 + (y2 - y1) / e,
		y2p = (y1 + y2) / 2.0 + (x1 - x2) / e,
		x3p = (x1 + 2.0 * x2) / 3.0,
		y3p = (y1 + 2.0 * y2) / 3.0;

	drawLine(target, x1, y1, x1p, y1p);
	drawLine(target, x1p, y1p, x2p, y2p);
	drawLine(target, x2p, y2p, x3p, y3p);
	drawLine(target, x3p, y3p, x2, y2);
}

void von_koch(sf::RenderTarget& target, int nb, double x1, double y1, double x2, double y2)
{	
	if(nb == 0) 
	{
		ligne_brisee(target, x1, y1, x2, y2);
	} 
	else
	{
		double e = 2.0 * sqrt(3.0);
		double
			x1p = (2.0 * x1 + x2) / 3.0,
			y1p = (2.0 * y1 + y2) / 3.0,
			x2p = (x1 + x2) / 2.0 + (y2 - y1) / e,
			y2p = (y1 + y2) / 2.0 + (x1 - x2) / e,
			x3p = (x1 + 2.0 * x2) / 3.0,
			y3p = (y1 + 2.0 * y2) / 3.0;

		von_koch(target, nb - 1, x1, y1, x1p, y1p);
		von_koch(target, nb - 1, x1p, y1p, x2p, y2p);
		von_koch(target, nb - 1, x2p, y2p, x3p, y3p);
		von_koch(target, nb - 1, x3p, y3p, x2, y2);
	}
}

int main()
{

    sf::RenderWindow window(sf::VideoMode(WINDOW_WIDTH, WINDOW_HEIGHT), "Courbe de von Koch");

	//Chargement fonte
	sf::Font font;
    if (!font.loadFromFile("../res/arial.ttf")) return EXIT_FAILURE;

	//Titre
    // sf::Text titre("PROGSV : Flocons", font, 150);
    // titre.setRotation(45);
    // titre.setColor(sf::Color(255,255,255,50));
    // titre.setPosition(100,-90);
    // titre.setStyle(sf::Text::Bold);

    while (window.isOpen())
    {
		//Evénements
        sf::Event event;
        while (window.pollEvent(event))
        {
            if (event.type == sf::Event::Closed)
            {
                window.close();
            }
		}
		
		//Dessin
        window.clear(sf::Color(0,102,102));
		//Titre
        //window.draw(titre);

		// une courbe de von Koch unique:
		von_koch(window, DEPTH, OFFSET, WINDOW_HEIGHT, WINDOW_WIDTH, OFFSET);
		
		// un ensemble de courbe reliée entre elles:

		/*
		von_koch(window, DEPTH, OFFSET, WINDOW_HEIGHT/2, WINDOW_WIDTH/2, OFFSET); 
		von_koch(window, DEPTH, WINDOW_WIDTH/2, OFFSET, WINDOW_WIDTH - OFFSET, WINDOW_HEIGHT/2);
		//von_koch(window, DEPTH, 400, 750, 50, 400);
		von_koch(window, DEPTH, WINDOW_WIDTH/2, WINDOW_HEIGHT-OFFSET, OFFSET, WINDOW_HEIGHT/2);
		//von_koch(window, DEPTH, 750, 400, 400, 750);
		von_koch(window, DEPTH, WINDOW_WIDTH-OFFSET, WINDOW_HEIGHT/2, WINDOW_WIDTH/2, WINDOW_HEIGHT-OFFSET);

		*/
        window.display();
    }
		
    return EXIT_SUCCESS;
}
